package co.hyperverge.hyperkyc_flutter;

import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import co.hyperverge.hyperkyc.data.models.DocFlowConfig;
import co.hyperverge.hyperkyc.data.models.HyperKycConfig;
import co.hyperverge.hyperkyc.data.models.HyperKycFlow;

public class FLHyperKyc {
    public static final String TAG = FLHyperKyc.class.getCanonicalName();

    public HyperKycConfig getHyperKycConfigFromMap(Map<String, Object> hyperKycConfigMap) {
        Log.e(TAG, "hyperKycConfigMap : " + hyperKycConfigMap);
        String appId = null;
        String appKey = null;
        String accessToken = null;
        String transactionId = null;
        List<HyperKycFlow> workFlow = null;
        String defaultCountryId = null;
        if (hyperKycConfigMap.containsKey("appId")) {
            appId = (String) hyperKycConfigMap.get("appId");
        }
        if (hyperKycConfigMap.containsKey("appKey")) {
            appKey = (String) hyperKycConfigMap.get("appKey");
        }
        if (hyperKycConfigMap.containsKey("accessToken")) {
            accessToken = (String) hyperKycConfigMap.get("accessToken");
        }
        if (hyperKycConfigMap.containsKey("transactionId")) {
            transactionId = (String) hyperKycConfigMap.get("transactionId");
        }
        if (hyperKycConfigMap.containsKey("defaultCountryId")) {
            defaultCountryId = (String) hyperKycConfigMap.get("defaultCountryId");
        }
        if (hyperKycConfigMap.containsKey("workFlow")) {
            List<Map> workflowList = (List<Map>) hyperKycConfigMap.get("workFlow");
            workFlow = getWorkflowFromList(workflowList);
        }

        if (accessToken == null) {
            assert appId != null;
            assert appKey != null;
            assert workFlow != null;
            assert transactionId != null;
            return new HyperKycConfig(appId, appKey, workFlow, transactionId, defaultCountryId);
        } else {
            assert workFlow != null;
            assert transactionId != null;
            return new HyperKycConfig(accessToken, workFlow, transactionId, defaultCountryId);
        }
    }

    private List<HyperKycFlow> getWorkflowFromList(List<Map> workflowList) {
        List<HyperKycFlow> hyperKycWorkFlow = new ArrayList<>();
        for (Map workflow : workflowList) {
            String type = (String) workflow.get("type");
            String countryId = (String) workflow.get("countryId");
            String documentId = (String) workflow.get("documentId");
            Boolean useForFaceMatch = (Boolean) workflow.get("useForFaceMatch");

            assert type != null;
            hyperKycWorkFlow.add(getWorkflow(type, countryId, documentId, useForFaceMatch));
        }
        return hyperKycWorkFlow;
    }

    private HyperKycFlow getWorkflow(String type, String countryId, String documentId, Boolean useForFaceMatch) {
        switch (type) {
            case "document":
                return new HyperKycFlow.Document(new DocFlowConfig(useForFaceMatch, countryId, documentId));
            case "face":
                return new HyperKycFlow.Face();
            default:
                return new HyperKycFlow.Document();
        }
    }
}
