//
//  FLHyperKyc.swift
//  hyperkyc_flutter
//
//  Created by Shrirama Upadhya A on 22/01/22.
//

import Foundation
import HyperKYC

class FLHyperKyc {
    
    public static func launch(context: UIViewController, hyperKycConfigArgument : [String: Any], callback: @escaping ((HyperKycResult)->Void)){
        
        let appId = hyperKycConfigArgument["appId"] as? String
        let appKey = hyperKycConfigArgument["appKey"] as? String
        let accessToken = hyperKycConfigArgument["accessToken"] as? String
        let transactionId = hyperKycConfigArgument["transactionId"] as! String
        let defaultCountryId = hyperKycConfigArgument["defaultCountryId"] as? String
        
        let workFlowArgument = hyperKycConfigArgument["workFlow"] as! [[String: Any]]
        let workflow = getWorkflowFromList(workflowList: workFlowArgument)
        
        if let accessToken = accessToken {
            let hyperKycConfig = HyperKycConfig(accessToken: accessToken, workFlow: workflow, transactionId: transactionId, defaultCountryId: defaultCountryId)
            HyperKyc.launch(context, hyperKycConfig: hyperKycConfig, callback)
        } else {
            let hyperKycConfig = HyperKycConfig(appId: appId!, appKey: appKey!, workFlow: workflow, transactionId: transactionId, defaultCountryId: defaultCountryId)
            HyperKyc.launch(context, hyperKycConfig: hyperKycConfig, callback)
        }
    }
    
    static func getWorkflowFromList(workflowList: [[String : Any]]) -> [HyperKycFlow] {
        var workFlow = [HyperKycFlow]()
        for workFlowComponent in workflowList {
            let type = workFlowComponent["type"] as! String
            let countryId = workFlowComponent["countryId"] as? String
            let documentId = workFlowComponent["documentId"] as? String
            let useForFaceMatch = workFlowComponent["useForFaceMatch"] as? Bool
            workFlow.append(getWorkflow(type: type, countryId: countryId, documentId: documentId, useForFaceMatch: useForFaceMatch))
        }
        return workFlow
    }
    
    static func getWorkflow(type: String, countryId: String?, documentId: String?, useForFaceMatch: Bool?) -> HyperKycFlow {
        switch type {
        case "document":
            return HyperKycFlow.document(config: DocFlowConfig(useForFaceMatch: useForFaceMatch ?? false, countryId: countryId, documentId: documentId))
        case "face":
            return HyperKycFlow.face()
        default : return HyperKycFlow.document()
        }
    }
    
}
