import 'package:hyperkyc_flutter/hyperkyc_doc_data.dart';
import 'package:hyperkyc_flutter/hyperkyc_face_data.dart';
import 'package:hyperkyc_flutter/hyperkyc_face_match_data.dart';

import 'hyperkyc_flutter_constants.dart';

/// HyperKycData will contain partial or the final result data based on the result state.
class HyperKycData {
  /// Selected country name
  final String? selectedCountry;

  /// List of document capture and OCR results
  final List<DocResult?>? docResultList;

  /// Face capture and liveness result
  final FaceData? faceData;

  /// List of Face match data results
  final List<FaceMatchDataResult?>? faceMatchDataResultList;

  /// Constructor
  HyperKycData({
    this.selectedCountry,
    this.docResultList,
    this.faceData,
    this.faceMatchDataResultList,
  });

  /// Convert HyperKycData object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperKycConstants.selectedCountry: this.selectedCountry,
      HyperKycConstants.docResultList: this.docResultList,
      HyperKycConstants.faceData: this.faceData,
      HyperKycConstants.faceMatchDataResultList: this.faceMatchDataResultList,
    };
  }

  /// Convert Map to HyperKycData object
  factory HyperKycData.fromMap(Map<dynamic, dynamic> map) {
    if (map.isEmpty) {
      return HyperKycData();
    }

    List<DocResult> docResultList = List.empty(growable: true);
    List docResultListFromMap = map[HyperKycConstants.docResultList] ?? [];
    for (var docResultMap in docResultListFromMap) {
      DocResult docResult = DocResult.fromMap(docResultMap ?? {});
      docResultList.add(docResult);
    }

    List<FaceMatchDataResult> faceMatchDataResultList =
        List.empty(growable: true);
    List faceMatchDataResultListFromMap =
        map[HyperKycConstants.faceMatchDataResultList] ?? [];
    for (var faceMatchResultMap in faceMatchDataResultListFromMap) {
      FaceMatchDataResult faceMatchDataResult =
          FaceMatchDataResult.fromMap(faceMatchResultMap ?? {});
      faceMatchDataResultList.add(faceMatchDataResult);
    }
    return HyperKycData(
      selectedCountry: map[HyperKycConstants.selectedCountry] as String?,
      docResultList: docResultList,
      faceData: FaceData.fromMap(map[HyperKycConstants.faceData] ?? {}),
      faceMatchDataResultList: faceMatchDataResultList,
    );
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycData{selectedCountry: $selectedCountry, docResultList: $docResultList, faceData: $faceData, faceMatchDataResultList: $faceMatchDataResultList}';
  }
}
